package com.xixun.xixunplayer;

import static android.graphics.Paint.Join.ROUND;
import static android.graphics.Paint.Style.FILL_AND_STROKE;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Path;
import android.graphics.Rect;
import android.graphics.RectF;
import android.view.Choreographer;
import android.view.View;

import java.time.LocalTime;
import java.time.ZoneId;

import gnph.util.JSMap;

public class EleAnaClock extends View implements Choreographer.FrameCallback {

    ZoneId timeZone;
    Bitmap img;
    int pinHourColor, pinMinColor, pinSecColor;
    Path hPath, mPath, sPath;
    float hAngle, mAngle, sAngle;
    Paint paint = new Paint();
    boolean showSecHand;

    public EleAnaClock(float w, float h, String path, JSMap layer, Context context) {
        super(context);
        timeZone = ZoneId.of(layer.stnn("timeZone"));
        img = BitmapFactory.decodeFile(path);
        pinHourColor = Color.parseColor(layer.stnn("pinHourColor"));
        pinMinColor = Color.parseColor(layer.stnn("pinMinColor"));
        pinSecColor = Color.parseColor(layer.stnn("pinSecColor"));
        var hhLen = layer.intg("pinHourLen");
        var mhLen = layer.intg("pinMinLen");
        var shLen = layer.intg("pinSecLen");
        var hhWidth = layer.intg("pinHourWidth");
        var mhWidth = layer.intg("pinMinWidth");
        var shWidth = layer.intg("pinSecWidth");
        showSecHand = layer.bool("showSecond");
        var sideLen = w;
        if(h < sideLen) sideLen = h;
        var lineWidth = sideLen/128;
        if(lineWidth < 1) lineWidth = 1;

        paint.setAntiAlias(true);
        paint.setStyle(FILL_AND_STROKE);
        paint.setStrokeWidth(lineWidth);
        paint.setStrokeJoin(ROUND);

        var rx = hhWidth*sideLen/400;
        hPath = new Path();
        hPath.moveTo(rx, 0);
        hPath.lineTo(0, hhLen*sideLen/-200);
        hPath.lineTo(-rx, 0);
        hPath.close();

        rx = mhWidth*sideLen/400;
        mPath = new Path();
        mPath.moveTo(rx, 0);
        mPath.lineTo(0, mhLen*sideLen/-200);
        mPath.lineTo(-rx, 0);
        mPath.close();
        rx = shWidth*sideLen/400;
        sPath = new Path();
        sPath.moveTo(rx, 0);
        sPath.lineTo(0, shLen*sideLen/-200);
        sPath.lineTo(-rx, 0);
        sPath.close();
    }
    void cal() {
        var time = LocalTime.now(timeZone);
        sAngle = time.getSecond() * 6;
        mAngle = time.getMinute() * 6 + sAngle/60;
        hAngle = time.getHour() * 30 + mAngle/12;
    }
    @Override
    protected void onDraw(Canvas canvas) {
        super.onDraw(canvas);
        if(lastSec==0) {
            cal();
            choreographer.postFrameCallback(this);
        }
        if(img != null) canvas.drawBitmap(img, new Rect(0,0,img.getWidth(), img.getHeight()), new RectF(0,0, getWidth(), getHeight()), paint);
        canvas.translate(getWidth()/2, getHeight()/2);

        paint.setColor(pinHourColor);
        canvas.rotate(hAngle);
        canvas.drawPath(hPath, paint);

        paint.setColor(pinMinColor);
        canvas.rotate(mAngle-hAngle);
        canvas.drawPath(mPath, paint);
        if(showSecHand) {
            paint.setColor(pinSecColor);
            canvas.rotate(sAngle-mAngle);
            canvas.drawPath(sPath, paint);
        }
    }

    Choreographer choreographer = Choreographer.getInstance();
    long lastSec = 0;

    @Override
    public void doFrame(long frameTimeNanos) {
        if(! isShown()) {
            lastSec = 0;
            return;
        }
        var sec = System.currentTimeMillis() / 1000;
        if(sec != lastSec) {
            lastSec = sec;
            cal();
            invalidate();
        }
        choreographer.postFrameCallback(this);
    }
}